@@ -483,6 +483,7 @@ token_info_build(sen_index *i, const char *string, size_t string_len, token_info
       break;
     }
     tid = sen_lex_next(lex);
+    if (lex->force_prefix) { ef |= EX_PREFIX; }
     switch (lex->status) {
     case sen_lex_doing :
       ti = token_info_open(i, _sen_sym_key(i->lexicon, tid), lex->pos, ef & EX_SUFFIX);
@@ -683,7 +684,7 @@ sen_index_create(const char *path, int key_size,
   }
   if (!(i = SEN_MALLOC(sizeof(sen_index)))) { return NULL; }
   SEN_LOG(sen_log_notice, "creating '%s' encoding=%s initial_n_segments=%d",
-          path, sen_enc_string[encoding], initial_n_segments);
+          path, sen_enctostr(encoding), initial_n_segments);
   strcpy(buffer, path);
   strcat(buffer, ".SEN");
   i->foreign_flags = 0;
@@ -777,7 +778,7 @@ sen_index_create_with_keys(const char *path, sen_sym *keys,
   }
   if (!(i = SEN_MALLOC(sizeof(sen_index)))) { return NULL; }
   SEN_LOG(sen_log_notice, "creating '%s' encoding=%s initial_n_segments=%d",
-          path, sen_enc_string[encoding], initial_n_segments);
+          path, sen_enctostr(encoding), initial_n_segments);
   i->keys = keys;
   i->foreign_flags = FOREIGN_KEY;
   strcpy(buffer, path);
@@ -860,7 +861,7 @@ sen_index_create_with_keys_lexicon(const char *path, sen_sym *keys, sen_sym *lex
   if (initial_n_segments == 0) { initial_n_segments = sen_initial_n_segments; }
   if (!(i = SEN_MALLOC(sizeof(sen_index)))) { return NULL; }
   SEN_LOG(sen_log_notice, "creating '%s' encoding=%s initial_n_segments=%d",
-          path, sen_enc_string[lexicon->encoding], initial_n_segments);
+          path, sen_enctostr(lexicon->encoding), initial_n_segments);
   i->keys = keys;
   i->lexicon = lexicon;
   i->foreign_flags = FOREIGN_KEY|FOREIGN_LEXICON;
@@ -868,7 +869,6 @@ sen_index_create_with_keys_lexicon(const char *path, sen_sym *keys, sen_sym *lex
   if ((i->inv = sen_inv_create(path, i->lexicon, initial_n_segments))) {
     return i;
   }
-  sen_inv_close(i->inv);
   SEN_FREE(i);
   return NULL;
 }
@@ -890,7 +890,6 @@ sen_index_open_with_keys_lexicon(const char *path, sen_sym *keys, sen_sym *lexic
   if ((i->inv = sen_inv_open(path, i->lexicon))) {
     return i;
   }
-  sen_inv_close(i->inv);
   SEN_FREE(i);
   return NULL;
 }
