@@ -1026,8 +1026,7 @@ buffer_flush(sen_inv *inv, uint32_t seg, sen_set *h)
 /* inv */
 
 sen_inv *
-sen_inv_create08(const char *path,  sen_sym *lexicon, 
-               int flags, uint32_t initial_n_segments)
+sen_inv_create08(const char *path, sen_sym *lexicon, uint32_t initial_n_segments)
 {
   int i, max_chunk;
   sen_io *seg, *chunk;
