@@ -76,9 +76,18 @@ inline static sen_rc sen_open(fileinfo *fi, const char *path, int flags, size_t
 inline static void sen_fileinfo_init(fileinfo *fis, int nfis);
 inline static int sen_opened(fileinfo *fi);
 inline static sen_rc sen_close(fileinfo *fi);
+#if defined(WIN32) && defined(WIN32_FMO_EACH)
+inline static void * sen_mmap(HANDLE *fmo, fileinfo *fi, off_t offset, size_t length);
+inline static int sen_munmap(HANDLE *fmo, void *start, size_t length);
+#define SEN_MMAP(fmo,fi,offset,length) (sen_mmap((fmo), (fi), (offset), (length)))
+#define SEN_MUNMAP(fmo,start,length) (sen_munmap((fmo), (start), (length)))
+#else /* defined(WIN32) && defined(WIN32_FMO_EACH) */
 inline static void * sen_mmap(fileinfo *fi, off_t offset, size_t length);
-inline static int sen_msync(void *start, size_t length);
 inline static int sen_munmap(void *start, size_t length);
+#define SEN_MMAP(fmo,fi,offset,length) (sen_mmap((fi), (offset), (length)))
+#define SEN_MUNMAP(fmo,start,length) (sen_munmap((start), (length)))
+#endif  /* defined(WIN32) && defined(WIN32_FMO_EACH) */
+inline static int sen_msync(void *start, size_t length);
 inline static sen_rc sen_pread(fileinfo *fi, void *buf, size_t count, off_t offset);
 inline static sen_rc sen_pwrite(fileinfo *fi, void *buf, size_t count, off_t offset);
 
@@ -101,8 +110,8 @@ sen_io_create(const char *path, uint32_t header_size, uint32_t segment_size,
     / SEN_IO_FILE_SIZE);
   if (!(fis = SEN_MALLOC(sizeof(fileinfo) * max_nfiles))) { return NULL; }
   sen_fileinfo_init(fis, max_nfiles);
-  if (!sen_open(fis, path, O_RDWR|O_CREAT|O_TRUNC, SEN_IO_FILE_SIZE)) {
-    if ((header = sen_mmap(fis, 0, b))) {
+  if (!sen_open(fis, path, O_RDWR|O_CREAT|O_TRUNC, SEN_IO_FILE_SIZE)) { 
+    if ((header = SEN_MMAP(&fis->fmo, fis, 0, b))) {
       header->header_size = header_size;
       header->segment_size = segment_size;
       header->max_segment = max_segment;
@@ -126,7 +135,7 @@ sen_io_create(const char *path, uint32_t header_size, uint32_t segment_size,
         }
         SEN_FREE(io);
       }
-      sen_munmap(header, b);
+      SEN_MUNMAP(&fis->fmo, header, b);
     }
     sen_close(fis);
   }
@@ -177,7 +186,7 @@ sen_io_open(const char *path, sen_io_mode mode, uint32_t cache_size)
   sen_fileinfo_init(fis, max_nfiles);
   if (!sen_open(fis, path, O_RDWR, SEN_IO_FILE_SIZE)) {
     io_header *header;
-    if ((header = sen_mmap(fis, 0, b))) {
+    if ((header = SEN_MMAP(&fis->fmo, fis, 0, b))) {
       if ((io = SEN_MALLOC(sizeof(sen_io)))) {
         if ((io->maps = SEN_CALLOC(sizeof(sen_io_mapinfo) * max_segment))) {
           strncpy(io->path, path, PATH_MAX);
@@ -203,7 +212,7 @@ sen_io_open(const char *path, sen_io_mode mode, uint32_t cache_size)
         }
         SEN_FREE(io);
       }
-      sen_munmap(header, b);
+      SEN_MUNMAP(&fis->fmo, header, b);
     }
     sen_close(fis);
   }
@@ -226,10 +235,10 @@ sen_io_close(sen_io *io)
   for (mi = io->maps, i = max_segment; i; mi++, i--) {
     if (mi->map) {
       /* if (atomic_read(mi->nref)) { return STILL_IN_USE ; } */
-      sen_munmap(mi->map, segment_size);
+      SEN_MUNMAP(&mi->fmo, mi->map, segment_size);
     }
   }
-  sen_munmap(io->header, io->base);
+  SEN_MUNMAP(&io->fis->fmo, io->header, io->base);
   for (fi = io->fis, i = max_nfiles; i; fi++, i--) { sen_close(fi); }
   SEN_FREE(io->fis);
   SEN_FREE(io->maps);
@@ -381,7 +390,7 @@ sen_io_win_map(sen_io *io, sen_io_win *iw, uint32_t segment,
           return NULL;
         }
       }
-      if (!(p = sen_mmap(fi, pos, nseg * segment_size))) {
+      if (!(p = SEN_MMAP(&iw->fmo, fi, pos, nseg * segment_size))) {
         return NULL;
       }
     } else {
@@ -587,7 +596,7 @@ sen_io_win_unmap(sen_io_win *iw)
   case sen_io_rdwr:
     // if (nseg > 1) { /* auto unmap is not implemented yet */
     if (nseg > 0) {
-      sen_munmap(((byte *)iw->addr) - iw->offset, nseg * segment_size);
+      SEN_MUNMAP(&iw->fmo, ((byte *)iw->addr) - iw->offset, nseg * segment_size);
     } else {
       rc = sen_io_seg_unref(io, iw->segment);
 #ifdef USE_AIO
@@ -641,13 +650,13 @@ sen_io_win_unmap(sen_io_win *iw)
     char path[PATH_MAX];\
     gen_pathname(io->path, path, fno);\
     if (!sen_open(fi, path, O_RDWR|O_CREAT, SEN_IO_FILE_SIZE)) {\
-      if ((info->map = sen_mmap(fi, pos, segment_size))) {\
+      if ((info->map = SEN_MMAP(&info->fmo, fi, pos, segment_size))) {\
         uint32_t nmaps;\
         SEN_ATOMIC_ADD_EX(&io->nmaps, 1, nmaps);\
       }\
     }\
   } else {\
-    if ((info->map = sen_mmap(fi, pos, segment_size))) {\
+    if ((info->map = SEN_MMAP(&info->fmo, fi, pos, segment_size))) {\
       uint32_t nmaps;\
       SEN_ATOMIC_ADD_EX(&io->nmaps, 1, nmaps);\
     }\
@@ -744,7 +753,7 @@ sen_io_seg_expire(sen_io *io, uint32_t segno)
         continue;
       } else {
         uint32_t nmaps;
-        sen_munmap(info->map, io->header->segment_size);
+        SEN_MUNMAP(&info->fmo, info->map, io->header->segment_size);
         info->map = NULL;
         SEN_ATOMIC_ADD_EX(pnref, -(SEN_IO_MAX_REF + 1), nref);
         SEN_ATOMIC_ADD_EX(&io->nmaps, -1, nmaps);
@@ -769,10 +778,80 @@ sen_get_pagesize(void)
   return (unsigned int)si.dwAllocationGranularity;
 }
 
+#ifdef WIN32_FMO_EACH
+
+inline static sen_rc
+sen_open(fileinfo *fi, const char *path, int flags, size_t maxsize)
+{
+  fi->fh = CreateFile(path, GENERIC_READ | GENERIC_WRITE,
+                      FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
+                      OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, 0);
+  if (fi->fh == INVALID_HANDLE_VALUE) { return sen_file_operation_error; }
+  if ((flags & O_TRUNC)) {
+    CloseHandle(fi->fh);
+    fi->fh = CreateFile(path, GENERIC_READ | GENERIC_WRITE,
+                        FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
+                        TRUNCATE_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
+    if (fi->fh == INVALID_HANDLE_VALUE) { return sen_file_operation_error; }
+  }
+  MUTEX_INIT(fi->mutex);
+  return sen_success;
+}
+
+inline static void *
+sen_mmap(HANDLE *fmo, fileinfo *fi, off_t offset, size_t length)
+{
+  void *res;
+
+  /* MUTEX_LOCK(fi->mutex); */
+  /* try to create fmo */
+  *fmo = CreateFileMapping(fi->fh, NULL, PAGE_READWRITE, 0, offset + length, NULL);
+  if (!*fmo) { return NULL; }
+  res = MapViewOfFile(*fmo, FILE_MAP_WRITE, 0, (DWORD)offset, (SIZE_T)length);
+  if (res) {
+    mmap_size += length;
+  } else {
+    SEN_LOG(sen_log_alert, "MapViewOfFile failed #%d", GetLastError());
+  }
+  /* MUTEX_UNLOCK(fi->mutex); */
+  return res;
+}
+
+inline static int
+sen_munmap(HANDLE *fmo, void *start, size_t length)
+{
+  int r = 0;
+  if (UnmapViewOfFile(start)) {
+    mmap_size -= length;
+  } else {
+    SEN_LOG(sen_log_alert, "munmap(%p,%d) failed <%d>", start, length, mmap_size);
+    r = -1;
+  }
+  if (*fmo) {
+    CloseHandle(*fmo);
+    *fmo = NULL;
+  }
+  return r;
+}
+
+inline static sen_rc
+sen_close(fileinfo *fi)
+{
+  if (fi->fmo != NULL) {
+    SEN_LOG(sen_log_alert, "file mapping object exists");
+  }
+  if (fi->fh != INVALID_HANDLE_VALUE) {
+    CloseHandle(fi->fh);
+    MUTEX_DESTROY(fi->mutex);
+    fi->fh = INVALID_HANDLE_VALUE;
+  }
+  return sen_success;
+}
+
+#else /* WIN32_FMO_EACH */
 inline static sen_rc
 sen_open(fileinfo *fi, const char *path, int flags, size_t maxsize)
 {
-  char *pd, *ps, buf[PATH_MAX];
   /* may be wrong if flags is just only O_RDWR */
   fi->fh = CreateFile(path, GENERIC_READ | GENERIC_WRITE,
                       FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
@@ -786,21 +865,17 @@ sen_open(fileinfo *fi, const char *path, int flags, size_t maxsize)
                         TRUNCATE_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
     if (fi->fh == INVALID_HANDLE_VALUE) { return sen_file_operation_error; }
   }
-  for (pd = buf, ps = path; *ps && pd < buf + PATH_MAX - 1; ps++, pd++) {
-    *pd = *ps == '\\' ? '_' : *ps;
-  }
-  *pd = '\0';
   /* signature may be wrong.. */
-  fi->fmo = OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, buf);
+  fi->fmo = OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, NULL);
   /* open failed */
   if (fi->fmo == NULL) {
     // flock
     /* retry to open */
-    fi->fmo = OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, buf);
+    fi->fmo = OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, NULL);
     /* failed again */
     if (fi->fmo == NULL) {
       /* try to create fmo */
-      fi->fmo = CreateFileMapping(fi->fh, NULL, PAGE_READWRITE, 0, SEN_IO_FILE_SIZE, buf);
+      fi->fmo = CreateFileMapping(fi->fh, NULL, PAGE_READWRITE, 0, SEN_IO_FILE_SIZE, NULL);
     }
     // funlock
   }
@@ -819,36 +894,6 @@ sen_open(fileinfo *fi, const char *path, int flags, size_t maxsize)
   return sen_file_operation_error;
 }
 
-inline static void
-sen_fileinfo_init(fileinfo *fis, int nfis)
-{
-  for (; nfis--; fis++) {
-    fis->fh = INVALID_HANDLE_VALUE;
-    fis->fmo = NULL;
-  }
-}
-
-inline static int
-sen_opened(fileinfo *fi)
-{
-  return fi->fh != INVALID_HANDLE_VALUE;
-}
-
-inline static sen_rc
-sen_close(fileinfo *fi)
-{
-  if (fi->fmo != NULL) {
-    CloseHandle(fi->fmo);
-    fi->fmo = NULL;
-  }
-  if (fi->fh != INVALID_HANDLE_VALUE) {
-    CloseHandle(fi->fh);
-    MUTEX_DESTROY(fi->mutex);
-    fi->fh = INVALID_HANDLE_VALUE;
-  }
-  return sen_success;
-}
-
 inline static void *
 sen_mmap(fileinfo *fi, off_t offset, size_t length)
 {
@@ -878,13 +923,6 @@ sen_mmap(fileinfo *fi, off_t offset, size_t length)
   return res;
 }
 
-inline static int
-sen_msync(void *start, size_t length)
-{
-  /* return value may be wrong... */
-  return FlushViewOfFile(start, length);
-}
-
 inline static int
 sen_munmap(void *start, size_t length)
 {
@@ -897,6 +935,44 @@ sen_munmap(void *start, size_t length)
   }
 }
 
+inline static sen_rc
+sen_close(fileinfo *fi)
+{
+  if (fi->fmo != NULL) {
+    CloseHandle(fi->fmo);
+    fi->fmo = NULL;
+  }
+  if (fi->fh != INVALID_HANDLE_VALUE) {
+    CloseHandle(fi->fh);
+    MUTEX_DESTROY(fi->mutex);
+    fi->fh = INVALID_HANDLE_VALUE;
+  }
+  return sen_success;
+}
+#endif /* WIN32_FMO_EACH */
+
+inline static void
+sen_fileinfo_init(fileinfo *fis, int nfis)
+{
+  for (; nfis--; fis++) {
+    fis->fh = INVALID_HANDLE_VALUE;
+    fis->fmo = NULL;
+  }
+}
+
+inline static int
+sen_opened(fileinfo *fi)
+{
+  return fi->fh != INVALID_HANDLE_VALUE;
+}
+
+inline static int
+sen_msync(void *start, size_t length)
+{
+  /* return value may be wrong... */
+  return FlushViewOfFile(start, length);
+}
+
 inline static sen_rc
 sen_pread(fileinfo *fi, void *buf, size_t count, off_t offset)
 {
