@@ -26,7 +26,11 @@ extern "C" {
 #endif
 
 #ifdef WIN32
+#ifdef WIN32_FMO_EACH
+#define SEN_IO_FILE_SIZE               1073741824UL
+#else /* FMO_EACH */
 #define SEN_IO_FILE_SIZE               134217728L
+#endif /* FMO_EACH */
 #else /* WIN32 */
 #define SEN_IO_FILE_SIZE               1073741824UL
 #endif /* WIN32 */
@@ -55,12 +59,18 @@ typedef struct {
   void *addr;
   uint32_t diff;
   int cached;
+#if defined(WIN32) && defined(WIN32_FMO_EACH)
+  HANDLE fmo;
+#endif /* defined(WIN32) && defined(WIN32_FMO_EACH) */
 } sen_io_win;
 
 typedef struct {
   void *map;
   uint32_t nref;
   uint32_t count;
+#if defined(WIN32) && defined(WIN32_FMO_EACH)
+  HANDLE fmo;
+#endif /* defined(WIN32) && defined(WIN32_FMO_EACH) */
 } sen_io_mapinfo;
 
 struct _sen_io {
