@@ -44,6 +44,7 @@ sen_ngram_open(sen_sym *sym, sen_nstr *nstr, uint8_t flags)
   lex->uni_alpha = (nstr->ctypes && !(lex->sym->flags & SEN_INDEX_SPLIT_ALPHA));
   lex->uni_digit = (nstr->ctypes && !(lex->sym->flags & SEN_INDEX_SPLIT_DIGIT));
   lex->uni_symbol = (nstr->ctypes && !(lex->sym->flags & SEN_INDEX_SPLIT_SYMBOL));
+  lex->force_prefix = 0;
   return lex;
 }
 
@@ -57,6 +58,7 @@ sen_ngram_next(sen_lex *lex)
   int32_t len = 0, pos;
   const unsigned char *p, *q, *r;
   if (lex->status) { return SEN_SYM_NIL; }
+  lex->force_prefix = 0;
   for (p = lex->next, pos = lex->pos + lex->skip; *p; p = r, pos++) {
     if (lex->nstr->ctypes) { cp = lex->nstr->ctypes + pos; }
     if (lex->uni_alpha && SEN_NSTR_CTYPE(*cp) == sen_str_alpha) {
@@ -177,6 +179,7 @@ sen_ngram_next(sen_lex *lex)
         if (blankp && !(lex->flags & SEN_LEX_UPD)) { continue; }
       }
       if ((!cl || !*q) && !(lex->flags & SEN_LEX_UPD)) { lex->status = sen_lex_done; }
+      if (len < SEN_LEX_NGRAM_UNIT_SIZE) { lex->force_prefix = 1; }
       {
         size_t blen = q - p;
         if (!blen) {
@@ -253,11 +256,13 @@ sen_mecab_open(sen_sym *sym, sen_nstr *nstr, uint8_t flags)
     sen_lex_close(lex);
     return NULL;
   }
+  // certain version of mecab returns trailing lf or spaces.
   for (p = buf + strlen(buf) - 1; *p == '\n' || isspace(*(unsigned char *)p); p--) { *p = '\0'; }
   //sen_log("sparsed='%s'", s);
   lex->orig = (unsigned char *)nstr->norm;
   lex->buf = (unsigned char *)buf;
   lex->next = (unsigned char *)buf;
+  lex->force_prefix = 0;
   return lex;
 }
 
@@ -273,7 +278,8 @@ sen_mecab_next(sen_lex *lex)
   if (lex->status) { return SEN_SYM_NIL; }
   for (p = lex->next, len = 0;;) {
     size_t cl;
-    if (!(cl = sen_str_charlen((char *)p, lex->encoding)) || isspace(*p)) {
+    if (!(cl = sen_str_charlen((char *)p, lex->encoding)) ||
+        sen_isspace(p, lex->encoding)) {
       break;
     }
     p += cl;
@@ -293,7 +299,8 @@ sen_mecab_next(sen_lex *lex)
   if (tid == SEN_SYM_NIL) {
     lex->status = sen_lex_not_found;
   } else {
-    while (isspace(*p)) { p++; }
+    int cl;
+    while ((cl = sen_isspace(p, lex->encoding))) { p += cl; }
     lex->next = p;
     lex->offset = offset;
     lex->len = len;
@@ -310,6 +317,7 @@ sen_mecab_next(sen_lex *lex)
 inline static sen_lex *
 sen_delimited_open(sen_sym *sym, sen_nstr *nstr, uint8_t flags)
 {
+  int cl;
   sen_lex *lex;
   const char *p;
   if (!(lex = SEN_MALLOC(sizeof(sen_lex)))) { return NULL; }
@@ -327,11 +335,12 @@ sen_delimited_open(sen_sym *sym, sen_nstr *nstr, uint8_t flags)
   lex->nstr = nstr;
   p = nstr->norm;
   lex->orig = (unsigned char *)p;
-  while (isspace(*p)) { p++; }
+  while ((cl = sen_isspace(p, lex->encoding))) { p += cl; }
   lex->next = (unsigned char *)p;
   lex->offset = 0;
   lex->len = 0;
   if (!*p) { lex->status = sen_lex_done; }
+  lex->force_prefix = 0;
   return lex;
 }
 
@@ -347,7 +356,8 @@ sen_delimited_next(sen_lex *lex)
   if (lex->status) { return SEN_SYM_NIL; }
   for (p = lex->next, len = 0;;) {
     size_t cl;
-    if (!(cl = sen_str_charlen((char *)p, lex->encoding)) || isspace(*p)) {
+    if (!(cl = sen_str_charlen((char *)p, lex->encoding)) ||
+        sen_isspace(p, lex->encoding)) {
       break;
     }
     p += cl;
@@ -367,7 +377,8 @@ sen_delimited_next(sen_lex *lex)
   if (tid == SEN_SYM_NIL) {
     lex->status = sen_lex_not_found;
   } else {
-    while (isspace(*p)) { p++; }
+    int cl;
+    while ((cl = sen_isspace(p, lex->encoding))) { p += cl; }
     lex->next = p;
     lex->offset = offset;
     lex->len = len;
