@@ -4,12 +4,12 @@
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
-  
+
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
-  
+
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -59,6 +59,7 @@ typedef struct {
   uint8_t uni_alpha;
   uint8_t uni_digit;
   uint8_t uni_symbol;
+  uint8_t force_prefix;
   sen_encoding encoding;
 } sen_lex;
 
@@ -69,7 +70,7 @@ enum {
 };
 
 sen_rc sen_lex_init(void);
-sen_lex *sen_lex_open(sen_sym *sym, const char *str, uint8_t flags);
+sen_lex *sen_lex_open(sen_sym *sym, const char *str, size_t str_len, uint8_t flags);
 sen_rc sen_lex_next(sen_lex *ng);
 sen_rc sen_lex_close(sen_lex *ng);
 sen_rc sen_lex_fin(void);
