@@ -22,7 +22,9 @@
 /* #define __USE_XOPEN2K 1 */
 #define _XOPEN_SOURCE 600
 #else
+#if !defined(__APPLE__) && !defined(__FreeBSD)
 #define _XOPEN_SOURCE 500
+#endif
 #endif /* USE_AIO */
 
 #ifdef HAVE_STDLIB_H
@@ -64,6 +66,7 @@
 #include <io.h>
 #include <basetsd.h>
 #include <process.h>
+#include <winsock2.h>
 #include <windows.h>
 #include <stddef.h>
 #include <windef.h>
@@ -82,6 +85,7 @@
 #define write _write
 #define close _close
 #define usleep Sleep
+#define sleep(x) Sleep((x) * 1000)
 #define uint8_t UINT8
 #define int8_t INT8
 #define int_least8_t INT8
@@ -90,6 +94,9 @@
 #define int32_t INT32
 #define uint32_t UINT32
 #define uint64_t UINT64
+#define ssize_t SSIZE_T
+#undef MSG_WAITALL
+#define MSG_WAITALL 0
 #define PATH_SEPARATOR "\\"
 #else /* WIN32 */
 #ifndef PATH_MAX
