@@ -41,6 +41,7 @@ entry_new(sen_set *set)
   if (set->garbages) {
     e = set->garbages;
     set->garbages = *((entry **)e);
+    memset(e, 0, set->entry_size);
   } else {
     byte *chunk = set->chunks[set->curr_chunk];
     if (!chunk) {
@@ -375,7 +376,11 @@ sen_set_cursor_next(sen_set_cursor *c, void **key, void **value)
 {
   uint32_t i;
   entry *e, **ep;
-  if (!c || !c->rest || c->index != c->set->index) { return NULL; }
+  if (!c || !c->rest) { return NULL; }
+  if (c->index != c->set->index) {
+    SEN_LOG(sen_log_alert, "sen_reset invoked during sen_set_cursor is opened!");
+    return NULL;
+  }
   for (i = c->rest, ep = c->curr;;i--, ep++) {
     if (!i) {
       c->rest = 0;
