@@ -372,7 +372,10 @@ sen_realloc(void *ptr, size_t size, const char* file, int line)
   void *res;
   res = realloc(ptr, size);
   if (res) { alloc_count++; }
-  if (!res) { SEN_LOG(sen_log_alert, "realloc fail (%p,%d)=%p (%s:%d) <%d>", ptr, size, res, file, line, alloc_count); }
+  if (size && !res) {
+    SEN_LOG(sen_log_alert, "realloc fail (%p,%d)=%p (%s:%d) <%d>",
+            ptr, size, res, file, line, alloc_count);
+  }
   return res;
 }
 
@@ -1428,7 +1431,7 @@ sen_nstr_close(sen_nstr *nstr)
   return sen_success;
 }
 
-const char *sen_enc_string[] = {
+static const char *sen_enc_string[] = {
   "default",
   "none",
   "euc_jp",
@@ -1438,6 +1441,16 @@ const char *sen_enc_string[] = {
   "koi8r"
 };
 
+const char *
+sen_enctostr(sen_encoding enc)
+{
+  if (enc < (sizeof(sen_enc_string) / sizeof(char *))) {
+    return sen_enc_string[enc];
+  } else {
+    return "unknown";
+  }
+}
+
 sen_encoding
 sen_strtoenc(const char *str)
 {
@@ -1481,13 +1494,13 @@ sen_isspace(const char *str, sen_encoding encoding)
   case '\v' :
     return 1;
   case 0x81 :
-    if (encoding == sen_enc_sjis && s[1] == 0x40) { return 1; }
+    if (encoding == sen_enc_sjis && s[1] == 0x40) { return 2; }
     break;
   case 0xA1 :
-    if (encoding == sen_enc_euc_jp && s[1] == 0xA1) { return 1; }
+    if (encoding == sen_enc_euc_jp && s[1] == 0xA1) { return 2; }
     break;
   case 0xE3 :
-    if (encoding == sen_enc_utf8 && s[1] == 0x80 && s[2] == 0x80) { return 1; }
+    if (encoding == sen_enc_utf8 && s[1] == 0x80 && s[2] == 0x80) { return 3; }
     break;
   default :
     break;
@@ -1563,60 +1576,135 @@ sen_str_itoa(int i, char *p, char *end, char **rest)
 }
 
 int
-sen_str_tok(char *str, const char *delim, char **tokbuf, int buf_size, char **rest)
+sen_str_tok(char *str, size_t str_len, char delim, char **tokbuf, int buf_size, char **rest)
 {
-  int n = 0;
-  while (n < buf_size) {
-    *tokbuf++ = str;
-    n++;
+  char **tok = tokbuf, **tok_end = tokbuf + buf_size;
+  if (buf_size > 0) {
+    char *str_end = str + str_len;
     for (;;str++) {
-      if (!*str) { goto exit; }
-      if (strchr(delim, *str)) { break; }
+      if (str == str_end) {
+        *tok++ = str;
+        break;
+      }
+      if (delim == *str) {
+        *tok++ = str;
+        if (tok == tok_end) { break; }
+      }
     }
-    *str++ = '\0';
   }
-exit :
   if (rest) { *rest = str; }
-  return n;
+  return tok - tokbuf;
 }
 
-#define UNIT_SIZE 4096
-
-sen_rc
-sen_rbuf_init(sen_rbuf *buf, int size)
+inline static void
+op_getopt_flag(int *flags, const sen_str_getopt_opt *o,
+               int argc, char * const argv[], int *i)
 {
-  if (!size) {
-    buf->head = NULL;
-    buf->curr = NULL;
-    buf->tail = NULL;
-  } else {
-    if (size < 0) { size = UNIT_SIZE; }
-    if (!(buf->head = SEN_REALLOC(NULL, size))) { return sen_memory_exhausted; }
-    buf->curr = buf->head;
-    buf->tail = buf->head + size;
+  switch (o->op) {
+    case getopt_op_none:
+      break;
+    case getopt_op_on:
+      *flags |= o->flag;
+      break;
+    case getopt_op_off:
+      *flags &= ~o->flag;
+      break;
+    case getopt_op_update:
+      *flags = o->flag;
+      break;
+    default:
+      return;
   }
-  return sen_success;
+  if (o->arg) {
+    if (++(*i) < argc) {
+      *o->arg = argv[*i];
+    } else {
+      /* TODO: error */
+    }
+  }
+}
+
+int
+sen_str_getopt(int argc, char * const argv[], const sen_str_getopt_opt *opts,
+               int *flags)
+{
+  int i;
+  for (i = 1; i < argc; i++) {
+    const char * v = argv[i];
+    if (*v == '-') {
+      const sen_str_getopt_opt *o;
+      int found;
+      if (*++v == '-') {
+        found = 0;
+        for (o = opts; o->opt != '\0' || o->longopt != NULL; o++) {
+          if (o->longopt && !strcmp(v, o->longopt)) {
+            op_getopt_flag(flags, o, argc, argv, &i);
+            found = 1;
+            break;
+          }
+        }
+        if (!found) { goto exit; }
+      } else {
+        const char *p;
+        for (p = v; *p; p++) {
+          found = 0;
+          for (o = opts; o->opt != '\0' || o->longopt != NULL; o++) {
+            if (o->opt && *p == o->opt) {
+              op_getopt_flag(flags, o, argc, argv, &i);
+              found = 1;
+              break;
+            }
+          }
+          if (!found) { goto exit; }
+        }
+      }
+    } else {
+      break;
+    }
+  }
+  return i;
+exit:
+  fprintf(stderr, "cannot recognize option '%s'.\n", argv[i]);
+  return -1;
+}
+
+#define UNIT_SIZE (1 << 12)
+#define UNIT_MASK (UNIT_SIZE - 1)
+
+sen_rc
+sen_rbuf_init(sen_rbuf *buf, size_t size)
+{
+  buf->head = NULL;
+  buf->curr = NULL;
+  buf->tail = NULL;
+  return size ? sen_rbuf_resize(buf, size) : sen_success;
 }
 
 sen_rc
-sen_rbuf_expand(sen_rbuf *buf, size_t size)
+sen_rbuf_resize(sen_rbuf *buf, size_t newsize)
 {
-  size_t newsize = (buf->tail - buf->head) + size;
-  char *head = SEN_REALLOC(buf->head, newsize);
-  if (!head) { return sen_memory_exhausted; }
-  buf->curr = head + (buf->curr - buf->head);
+  char *head;
+  newsize = (newsize + (UNIT_MASK)) & ~UNIT_MASK;
+  if (!(head = SEN_REALLOC(buf->head, newsize))) { return sen_memory_exhausted; }
+  buf->curr = head + SEN_RBUF_VSIZE(buf);
   buf->head = head;
   buf->tail = head + newsize;
   return sen_success;
 }
 
+sen_rc
+sen_rbuf_reinit(sen_rbuf *buf, size_t size)
+{
+  SEN_RBUF_REWIND(buf);
+  return sen_rbuf_resize(buf, size);
+}
+
 sen_rc
 sen_rbuf_write(sen_rbuf *buf, const char *str, size_t len)
 {
   sen_rc rc = sen_success;
-  if (buf->tail < buf->curr + len) {
-    size_t inc = (buf->curr + len - buf->tail + UNIT_SIZE - 1) & ~(UNIT_SIZE - 1);
-    if ((rc = sen_rbuf_expand(buf, inc))) { return rc; }
+  if (SEN_RBUF_REST(buf) < len) {
+    if ((rc = sen_rbuf_resize(buf, SEN_RBUF_VSIZE(buf) + len))) { return rc; }
   }
   memcpy(buf->curr, str, len);
   buf->curr += len;
@@ -1628,7 +1716,7 @@ sen_rbuf_itoa(sen_rbuf *buf, int i)
 {
   sen_rc rc = sen_success;
   while (sen_str_itoa(i, buf->curr, buf->tail, &buf->curr)) {
-    if ((rc = sen_rbuf_expand(buf, UNIT_SIZE))) { return rc; }
+    if ((rc = sen_rbuf_resize(buf, SEN_RBUF_WSIZE(buf) + UNIT_SIZE))) { return rc; }
   }
   return rc;
 }
