@@ -40,8 +40,6 @@ enum {
   sen_str_others
 };
 
-extern const char * sen_enc_string[];
-
 typedef struct {
   uint32_t ctype;
   char *start;
@@ -62,6 +60,21 @@ typedef struct {
   sen_encoding encoding;
 } sen_nstr;
 
+typedef enum {
+  getopt_op_none = 0,
+  getopt_op_on,
+  getopt_op_off,
+  getopt_op_update
+} sen_str_getopt_op;
+
+typedef struct {
+  const char opt; /* ends opt == 0 && longopt == NULL */
+  const char *longopt;
+  char **arg; /* if NULL, no arg are required */
+  int flag;
+  sen_str_getopt_op op;
+} sen_str_getopt_opt;
+
 sen_str_splitter *sen_str_splitter_open(char *str);
 char *sen_str_splitter_next(sen_str_splitter *s, int *ctype);
 sen_rc sen_str_splitter_close(sen_str_splitter *s);
@@ -93,10 +106,12 @@ int sen_isspace(const char *s, sen_encoding encoding);
 int sen_atoi(const char *nptr, const char *end, const char **rest);
 unsigned int sen_atoui(const char *nptr, const char *end, const char **rest);
 sen_rc sen_str_itoa(int i, char *p, char *end, char **rest);
+const char *sen_enctostr(sen_encoding enc);
 sen_encoding sen_strtoenc(const char *str);
 
 void sen_str_itoh(unsigned int i, char *p, unsigned int len);
-int sen_str_tok(char *str, const char *delim, char **tokbuf, int buf_size, char **rest);
+int sen_str_tok(char *str, size_t str_len, char delim, char **tokbuf, int buf_size, char **rest);
+int sen_str_getopt(int argc, char * const argv[], const sen_str_getopt_opt *opts, int *flags);
 
 #define SEN_MALLOC(s) sen_malloc(s,__FILE__,__LINE__)
 #define SEN_CALLOC(s) sen_calloc(s,__FILE__,__LINE__)
@@ -122,14 +137,19 @@ typedef struct {
   sen_lbuf_node **tail;
 } sen_lbuf;
 
-sen_rc sen_rbuf_init(sen_rbuf *buf, int size);
-sen_rc sen_rbuf_expand(sen_rbuf *buf, size_t size);
+sen_rc sen_rbuf_init(sen_rbuf *buf, size_t size);
+sen_rc sen_rbuf_reinit(sen_rbuf *buf, size_t size);
+sen_rc sen_rbuf_resize(sen_rbuf *buf, size_t newsize);
 sen_rc sen_rbuf_write(sen_rbuf *buf, const char *str, size_t len);
 sen_rc sen_rbuf_itoa(sen_rbuf *buf, int i);
 sen_rc sen_rbuf_fin(sen_rbuf *buf);
+
 #define SEN_RBUF_PUTS(buf,str) (sen_rbuf_write((buf), (str), strlen(str)))
 #define SEN_RBUF_PUTC(buf,c) { char _c = (c); sen_rbuf_write((buf), &_c, 1); }
 #define SEN_RBUF_REWIND(buf) ((buf)->curr = (buf)->head)
+#define SEN_RBUF_WSIZE(buf) ((buf)->tail - (buf)->head)
+#define SEN_RBUF_REST(buf) ((buf)->tail - (buf)->curr)
+#define SEN_RBUF_VSIZE(buf) ((buf)->curr - (buf)->head)
 
 sen_rc sen_lbuf_init(sen_lbuf *buf);
 void *sen_lbuf_add(sen_lbuf *buf, size_t size);
