@@ -1138,3 +1138,9 @@ sen_sym_next(sen_sym *sym, sen_id id)
   }
   return SEN_SYM_NIL;
 }
+
+sen_id
+sen_sym_curr_id(sen_sym *sym)
+{
+  return sym->header->curr_rec;
+}
