@@ -445,6 +445,19 @@ if (sen_logger_pass(SEN_LOG_DEFAULT_LEVEL)) {\
   sen_logger_put(SEN_LOG_DEFAULT_LEVEL, __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__);\
 }
 
+/* sen_com_client */
+
+typedef struct _sen_com_sqtp sen_com_client;
+
+sen_rc sen_com_init(void);
+void sen_com_fin(void);
+sen_com_client *sen_com_client_open(const char *host, int port);
+sen_rc sen_com_client_send(sen_com_client *c, const char *str, unsigned int str_len,
+                           int morep);
+sen_rc sen_com_client_recv(sen_com_client *c, char **str, unsigned int *str_len,
+                           int *morep);
+sen_rc sen_com_client_close(sen_com_client *c);
+
 #ifdef __cplusplus
 }
 #endif
