@@ -19,8 +19,6 @@
 #include <string.h>
 #include "sym.h"
 
-extern void sen_sym_inspect(sen_sym *sym);
-
 #define NUL             '\0'
 #define NL              '\n'
 
@@ -61,8 +59,6 @@ doit(int argc, char **argv)
     return;
   }
 
-  // sen_sym_inspect(sym);
-
   if (strcmp(argv[1], "get")) {
     fp = fopen(argv[2], "r");
     i = 1;
@@ -80,9 +76,6 @@ doit(int argc, char **argv)
     fclose(fp);
   }
 
-   sen_sym_inspect(sym);
-
-
   if (strcmp(argv[1], "put")) {
     fp = fopen(argv[2], "r");
     i = 1;
@@ -110,13 +103,10 @@ doit(int argc, char **argv)
     fclose(fp);
   }
 
-  sen_sym_inspect(sym);
-
   */
   {
     int j, n = sen_sym_size(sym);
     // printf("nrec=%d\n", n);
-    //    sen_sym_dump(sym, 20);
     for (i = n; i >= 1; i--) {
       sen_sym_key(sym, i, symbuf, 8192);
       j = sen_sym_at(sym, symbuf);
@@ -134,8 +124,6 @@ doit(int argc, char **argv)
       /*
       if (!(i % 10000)) {
         printf("deleting(%d) %s->%d\n", i, key, sen_sym_at(sym, key));
-        sen_sym_inspect(sym);
-        sen_sym_dump(sym, 10);
       }
       */
     }
@@ -148,7 +136,6 @@ doit(int argc, char **argv)
       }
     }
 
-    sen_sym_inspect(sym);
     puts("checked..");
 
     return;
@@ -183,7 +170,6 @@ doit(int argc, char **argv)
     printf("registered again.. i=%d, nrec=%d, nrec0=%d\n", i, sen_sym_size(sym), n);
 
   }
-  sen_sym_inspect(sym);
   sen_sym_close(sym);
 }
 
