@@ -16,6 +16,7 @@
 */
 #include "senna_in.h"
 #include <stdio.h>
+#include <string.h>
 #include "senna.h"
 
 #include "set.h"
@@ -29,6 +30,7 @@ main(int argc, char **argv)
   sen_query *q;
   sen_records *r;
   const char *rest;
+  unsigned int l;
   if (argc < 2) {
     fputs("usage: qtest indexname\n", stderr);
     return -1;
@@ -41,8 +43,8 @@ main(int argc, char **argv)
   while (!feof(stdin) && fgets(buf, 0x100000, stdin)) {
     r = sen_records_open(sen_rec_document, sen_rec_none, 0);
     q = sen_query_open(buf, strlen(buf), sen_sel_and, 10, sen_enc_euc_jp);
-    rest = sen_query_rest(q);
-    if (*rest) { printf("too long query rest.. \"%s\"\n", rest); }
+    l = sen_query_rest(q, &rest);
+    if (*rest) { printf("too long query rest.. \"%.*s\"\n", l, rest); }
     sen_query_exec(i, q, r, sen_sel_or);
     sen_query_close(q);
     printf("%d hits\n", sen_records_nhits(r));
